/* $Id: conf.c,v 1.18 2002/07/31 14:35:14 blues Exp $	*/
/* Borrowed from GNU Wget *
 * Modified by rawsock    */

/* ChangeLog {{{
 * $Log: conf.c,v $
 * Revision 1.18  2002/07/31 14:35:14  blues
 * - one TODO is out - it's possible now to turn off chats
 *
 * Revision 1.17  2002/07/20 16:38:17  blues
 * - final touch
 *
 * Revision 1.16  2002/07/20 16:29:39  blues
 * - helper descritptions for vim
 *
 * Revision 1.15  2002/06/30 06:44:10  blues
 * - more logging
 *
 * Revision 1.14  2002/06/30 04:04:08  blues
 * - added MOTD utility
 *
 * Revision 1.13  2002/06/30 02:34:52  blues
 * - clean-ups
 *
 * Revision 1.12  2002/06/30 02:22:15  blues
 * - debug renamed to daemon
 *
 * Revision 1.11  2002/06/30 02:03:10  blues
 * - even more cleanups
 * - UID and GID are now config options
 * - status expanded
 *
 * Revision 1.10  2002/06/29 23:55:33  blues
 * - missing changes
 *
 * Revision 1.9  2002/06/29 23:50:59  blues
 * - massive attack:
 * - new logging scheme - used Log Writer Library
 * - autoconf stuff adapted partially
 *
 * }}} */

/* Included headers {{{ */
#include <stdio.h>
#include <stdlib.h>
#include <syslog.h>
#include <lwl.h>
#include "main.h"
#include "pch.h"
/* }}} */

extern lwlh_t h;

/* Commands recognized by config {{{ */
const char *commands[]={"listening_port",
						"hubname",
						"user_limit",
						"allow_broken_key",
						"ping_timeout",
						"register_timeout",
						"nickname_pattern",
						"min_share",
						"daemon",
						"log",
						"user",
						"group",
						"motd",
						"allow_chat",
						""}; /* }}} */

/* strdupdelim() - Copy the string formed by two pointers to a new location {{{ */
char *strdupdelim(const char *beg,const char *end)
{
	char *res=(char *)malloc(end-beg+1);
	
	memcpy(res,beg,end-beg);
	res[end-beg]=0;
	return res;
} /* }}} */

/* Syntax: <sp>* command <sp>* = <sp>* value <newline>
 * Return values:
 *	1	- success
 *	0	- failure
 *	-1	- empty */ 
/* parse_line () {{{ */
int parse_line(const char *line,char **com,char **val)
{
	const char *p=line;
	const char *orig_comptr,*end;
	char *new_comptr;
	
	while(*p&&isspace(*p)) ++p;
	if (!*p||*p=='#') return -1;
	for (orig_comptr=p;isalpha(*p)||*p=='_'||*p=='-';p++);
	if (!isspace(*p)&&(*p!='=')) return 0;
	*com=(char *)malloc(p-orig_comptr+1);
	for (new_comptr=*com;orig_comptr<p;orig_comptr++)
	{ 
		//if (*orig_comptr=='_'||*orig_comptr=='-') continue;
		*new_comptr++ = *orig_comptr;
	}
	*new_comptr=0;
	for (;isspace(*p);p++);
	if (*p!='=') { free (*com); return 0; }
	for (++p;isspace(*p);p++);
	end=line+strlen(line)-1;
	while(end>p&&isspace(*end)) --end;
	*val=strdupdelim(p,end+1);
	return 1;
} /* }}} */

/* read_whole_line() - Read a line from FP and return the pointer to allocated storage. {{{ */
char *read_whole_line(FILE *fp)
{
	int length=0;
	int bufsize=82;
	char *line=(char *)malloc(bufsize);
	
	while(fgets(line+length,bufsize-length,fp))
	{
		length+=strlen(line+length);
		if (length==0) continue;
		if (line[length-1]=='\n') break;
		bufsize<<=1;
		line=realloc(line,bufsize);
	}
	if (length==0||ferror(fp)) { free (line); return NULL; }
	if (length+1<bufsize) line=realloc(line,length+1);
	return line;
} /* }}} */

/* parse_config() - Parse config file {{{ */
void parse_config(struct config *conf)
{
	char *line;
	int ln;
	FILE *fp;
	extern lwlh_t h;
	
	/* We look for config in location taken from configure
	 * If not found - take default values from initialization
	 * Opening config {{{ */
	fp=fopen(conf->file,"rb");
	if (!fp) {
		puts("Can't open config file!!");
		lwl_write_log (h, LWL_PRI_WARNING, "Can't read config %s: %s", conf->file, strerror(errno));
		lwl_write_log (h, LWL_PRI_WARNING, "Using default config values");
		return;
	}
	lwl_write_log (h, LWL_PRI_INFO, "Reading config file %s", conf->file);
	/* }}} */
	ln=1;
	while(line=read_whole_line(fp))
	{
		char *com,*val;
		int status;
		
		status=parse_line(line,&com,&val);
		free(line);
		if (status==1)
		{
			status=0;
			while(*commands[status])
			{ 
				if (!strcasecmp(com,commands[status])) break;
				else status++;
			}
			switch(status) /* Options {{{ */
			{
				/* listening_port {{{ */
				case 0: if ((status=atoi(val))==0) {
						fprintf(stderr,"Error in %s at line %d.\n",conf->file,ln);
						lwl_write_log (h, LWL_PRI_WARNING, "Error in %s at line %d: %s.",conf->file,ln,commands[status]);
						break;
					}
					conf->listen_port=status;
					break; /* }}} */
				/* hubname {{{ */
				case 1: if (conf->hubname) free(conf->hubname);
						conf->hubname=(char *)malloc(strlen(val)+1);
					strcpy(conf->hubname,val);
					break; /* }}} */
				/* user_limit {{{ */
				case 2: conf->userlimit=atoi(val);
					break; /* }}} */
				/* allow_broken_key {{{ */
				case 3: conf->allow_broken_key=atoi(val);
						break; /* }}} */
				/* ping_timeout {{{ */
				case 4: conf->ping_timeout=atoi(val);
					break; /* }}} */
				/* register_timeout {{{ */
				case 5: conf->register_timeout=atoi(val);
					break; /* }}} */
				/* nickname_pattern {{{ */
				case 6: if (conf->nick_pattern) free(conf->nick_pattern);
						conf->nick_pattern=(char *)malloc(strlen(val)+1);
					strcpy(conf->nick_pattern,val);
					break; /* }}} */
				/* min_share {{{ */
				case 7: conf->minshare=atoi(val);
					break; /* }}} */
				/* fork or not - daemon mode	- not tested... {{{ */
				case 8: conf->daemon=atoi(val);
					break; /* }}} */
				/* location of log file		- not tested... {{{ */
				case 9: if (conf->log) free(conf->log);
						conf->log==(char *)malloc(strlen(val)+1);
					strcpy(conf->log,val);
					break; /* }}} */
				/* UID to run {{{ */
				case 10: conf->uid_num=atoi(val);
					 break; /* }}} */
				/* GID to run {{{ */
				case 11: conf->gid_num=atoi(val);
					 break; /* }}} */
				/* Message of the day {{{ */
				case 12: if (conf->motd) free(conf->motd);
							 conf->motd=(char *)malloc(strlen(val)+1);
						 strcpy(conf->motd,val);
						 break; /* }}} */
				/* Allow chat {{{ */
				case 13: conf->allow_chat=atoi(val);
						 break; /* }}} */
				/* action when everything fails: {{{ */
				default: fprintf(stderr,"Error in %s at line %d.\n",conf->file,ln);
					lwl_write_log (h, LWL_PRI_WARNING, "Error in %s at line %d: %s.",conf->file,ln,commands[status]);
						 /* }}} */
			} /* }}} */
			free(com);
			free(val);
		}
		else if (status==0) 
		{
			fprintf(stderr,"Error in %s at line %d.\n",conf->file,ln);
			lwl_write_log (h, LWL_PRI_WARNING, "Error in %s at line %d.",conf->file,ln);
		}
		++ln;
	}
	fclose(fp);
} /* }}} */




/* VIM Settings {{{
 * Local variables:
 * tab-width: 4
 * c-basic-offset: 4
 * soft-stop-width: 4
 * c indent on
 * End:
 * vim600: sw=4 ts=4 sts=4 cindent fdm=marker
 * vim<600: sw=4 ts=4
 * }}} */

