// $Id: debug.h,v 1.11 2002/07/30 16:15:10 blues Exp $

/* Intro {{{
 * ---------------------------------------------------------------- 
 *  DConnect Daemon for Linux
 * 
 *      #(@) Copyright (c) 2002, DConnect development team
 *      #(@) Homepage: http://www.dc.ds.pg.gda.pl/
 *
 *  This  program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License as
 *  published by the Free Software Foundation; either version 2 of
 *  the License.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY  or  FITNESS FOR A PARTICULAR PURPOSE. See the
 *  GNU General Public License for more details.
 *
 *  You  should  have  received a copy  of  the GNU General Public
 *  License  along  with  this  program;  if  not,  write  to  the
 *  Free  Software  Foundation,  Inc.,  675  Mass  Ave,  Cambridge,
 *  MA 02139, USA. 
 *
 * ---------------------------------------------------------------- 
 * }}} */

/* ChangeLog {{{
 * $Log: debug.h,v $
 * Revision 1.11  2002/07/30 16:15:10  blues
 * - all files have now vim helpers - in *.h files only Intro and ChangeLog
 *   are hidden. In *.c files all functions are hidden.
 * - added "/" at the end of address, fixed missing.
 *
 * Revision 1.10  2002/06/30 07:51:00  blues
 * - switched to our main web page
 *
 * Revision 1.9  2002/06/30 06:49:03  blues
 * - cosmetics
 *
 * Revision 1.8  2002/06/06 11:00:41  blues
 * - add new line at the end of the file - causes warnings in RH
 *
 * Revision 1.7  2002/05/18 20:03:53  ventox
 * - some errors corrected
 *
 * Revision 1.6  2002/04/28 12:36:19  sebo
 * Daemon mode now works (I hope) and is the default. `-d' debug mode introduced.
 *
 * Revision 1.1  2002/04/27 18:21:10  ventox
 * Initial revision
 * }}} */

#if !defined( __vXhz__DEBUG_H__INCLUDED__ )
#define __vXhz__DEBUG_H__INCLUDED__

#if !defined( __vXhz__CONFIG_H__INCLUDED__ )
#error "config.h must be included before any local header file"
#endif /* _vXhz__CONFIG_H__INCLUDED__ */

/* declarations & definitions for debug purpose only */
#if defined( _DEBUG )

/***********************************************************************/
/* function:                                                           */
/*   void _vdebug_output_string( _vXhz_CONST char * fmt, ... )         */
/* arguments:                                                          */
/*   fmt - format control string (see printf function's format         */
/*         specification fields description for details)               */ 
/* description:                                                        */
/*   functions sends a formated string to the debuger. if there's no   */
/*   system debugger the output is redirected to the stderr stream     */
/*   or any other specyfied by function _vdebug_set_outstream function */ 
/***********************************************************************/

_vXhz_LIB_ENTRY void _vdebug_output_string( _vXhz_CONST char *, ... );

BOOL _vdebug_report( _vXhz_CONST char *, int );

#define THIS_FILE         __FILE__
#define ASSERT( f ) \
do { \
	if ( !(f) && _vdebug_report( THIS_FILE, __LINE__ ) ) \
		_vdebug_break(); \
} while ( FALSE ) \

#define VERIFY( f )       ASSERT( f )

#define DEBUG_ONLY( f )   (f)
#define DEBUG_T           _vdebug_output_string

typedef void (_vXhz_STDCALL * _vdebug_dump_proc)( FILE * );
void _vdebug_dump( _vdebug_dump_proc * );

#if defined( WIN32 )
/* by default it's "int 3" (if intel x86 machine), 
 * otherwise call DebugBreak instead...	*/
#if defined( _M_IX86 )
#define _vdebug_break() _asm { int 3 }
#else /* _M_IX86 */
#define _vdebug_break() DebugBreak()
#endif /* !_M_IX86 */
#else /* WIN32 */
/* ... or signal SIGABRT for Unix */
#define _vdebug_break() raise( SIGABRT )
#endif /* !WIN32 */

#else /* _DEBUG */

#define ASSERT( f )       ((void)0)
#define VERIFY( f )       ((void)(f))

/*_vXhz_INLINE void _vdebug_output_string( _vXhz_CONST char * fmt, ... ) { }  */

#define DEBUG_ONLY( f )   ((void)0)
#define DEBUG_T           1 ? ((void)0) : _vdebug_output_string

#undef  _vdebug_break
#define _vdebug_break()

#endif /* !_DEBUG */

#endif /* __vXhz__DEBUG_H__INCLUDED__ */



/* VIM Settings {{{
 * Local variables:
 * tab-width: 4
 * c-basic-offset: 4
 * soft-stop-width: 4
 * c indent on
 * End:
 * vim600: sw=4 ts=4 sts=4 cindent fdm=marker
 * vim<600: sw=4 ts=4
 * }}} */

