static char rcs_id[] =
    "$Id: list.c,v 1.7 2002/07/30 16:15:10 blues Exp $";

/* Intro {{{
 * ---------------------------------------------------------------- 
 *  DConnect Daemon for Linux
 * 
 *      #(@) Copyright (c) 2002, DConnect development team
 *      #(@) Homepage: http://www.dc.ds.pg.gda.pl/
 *
 *  This  program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License as
 *  published by the Free Software Foundation; either version 2 of
 *  the License.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY  or  FITNESS FOR A PARTICULAR PURPOSE. See the
 *  GNU General Public License for more details.
 *
 *  You  should  have  received a copy  of  the GNU General Public
 *  License  along  with  this  program;  if  not,  write  to  the
 *  Free  Software  Foundation,  Inc.,  675  Mass  Ave,  Cambridge,
 *  MA 02139, USA. 
 *
 * ---------------------------------------------------------------- 
 *  }}} */

/* ChangeLog {{{
 * $Log: list.c,v $
 * Revision 1.7  2002/07/30 16:15:10  blues
 * - all files have now vim helpers - in *.h files only Intro and ChangeLog
 *   are hidden. In *.c files all functions are hidden.
 * - added "/" at the end of address, fixed missing.
 *
 * Revision 1.6  2002/07/30 15:37:42  blues
 * - helpers for vim
 *
 * Revision 1.5  2002/06/30 07:51:00  blues
 * - switched to our main web page
 *
 * Revision 1.4  2002/06/30 07:01:47  blues
 * - cosmetics
 *
 * Revision 1.3  2002/05/18 20:03:53  ventox
 * - some errors corrected
 *
 * Revision 1.1  2002/04/27 18:21:10  ventox
 * Initial revision
 * }}} */

// include for PCH file
#include "pch.h"

// include local header files
#include "list.h"

#if defined( _DEBUG )
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif /* _DEBUG */

/* _vlist_create() {{{ */
_status_t _vlist_create( _vlist ** vl, _int32b threshold ) {
	ASSERT( vl );
	
	*vl = (_vlist *)malloc( sizeof( _vlist ) );
	if ( *vl == NULL )      
		return out_of_memory;
	
	(*vl)->_position  = 0;
	(*vl)->_size      = threshold;
	(*vl)->_threshold = threshold;
	(*vl)->_seek      = 0;
	
	(*vl)->_items     = 
		(void *)malloc( sizeof( void * ) * (*vl)->_threshold );
	
	if ( (*vl)->_items == NULL )
		return out_of_memory;
	
	memset( (*vl)->_items, 0, sizeof( void * ) * threshold );
	return OK;
} /* }}} */

/* _vlist_destroy() {{{ */
_status_t _vlist_destroy( _vlist ** vl, void (* clean_up)( void * ) ) {
	ASSERT( vl );
	
	if ( clean_up ) 
	{
		int i = 0;
		while ( _vlist_get_number_of_items( *vl ) > i )
			clean_up( _vlist_get_item_at( *vl, i++ ) );
	}
	
	DELETE_PTR( *vl );
	return OK;
} /* }}} */

#if defined( _DEBUG )
/* _vlist_is_empty() returns TRUE if there's no elements on the list {{{ */
BOOL _vlist_is_empty( _vlist * vl ) { 
	ASSERT( vl ); 
	return vl->_position == 0; 
} /* }}} */
#endif /* _DEBUG */

#if defined( _DEBUG )
/* _vlist_get_number_of_items() - gives total number of elements in the list {{{ */
_int32b _vlist_get_number_of_items( _vlist * vl ) {
	ASSERT( vl );
	return vl->_position;
} /* }}} */
#endif /* !_DEBUG */

/* _vlist_get_first_item() {{{ */
void * _vlist_get_first_item( _vlist * vl ) {
	ASSERT( vl );
	
	if ( vl->_position > 0 )
		return *(vl->_items);
	
	return NULL;
} /* }}} */

/* _vlist_get_last_item() {{{ */  
void * _vlist_get_last_item( _vlist * vl ) {
	ASSERT( vl );
	
	if ( vl->_position > 0 )
		return vl->_items[vl->_position - 1];
	
	return NULL;
} /* }}} */

#if defined( _DEBUG )
/* _vlist_get_seek_position() {{{ */
_int32b _vlist_get_seek_position( _vlist * vl ) {
	ASSERT( vl != NULL );
	return vl->_seek;
} /* }}} */
#endif /* _DEBUG */

/* _vlist_next() {{{ */
void * _vlist_next( _vlist * vl ) {
	void * result = NULL;    
	ASSERT( vl );
	
	if ( vl->_position == 0 ) 
		return NULL;
	
	if ( (vl->_seek + 1) < vl->_position ) {
		result = _vlist_get_at_seek_position( vl );      
		vl->_seek++;      
	}
	
	return result;
} /* }}} */

/* _vlist_previous() {{{ */
void * _vlist_previous( _vlist * vl ) {
	void * result = NULL;
	ASSERT( vl );
	
	if ( vl->_position == 0 )
		return NULL;
	
	if ( (vl->_seek - 1) >= 0 )
	{
		result = _vlist_get_at_seek_position( vl );      
		vl->_seek--;
	}
	
	return result;
} /* }}} */

#if defined( _DEBUG )  
/* _vlist_get_item_at() {{{ */
void * _vlist_get_item_at( _vlist * vl, _int32b index ) {   
	ASSERT( vl );
	ASSERT( (index < vl->_position) && (index >= 0) );
	
	return vl->_items[index];
} /* }}} */
#endif /* _DEBUG */

/* _vlist_set_item_at() {{{ */
void _vlist_set_item_at( _vlist * vl, _vXhz_CONST void * item, _int32b index ) {
	ASSERT( vl );
	ASSERT( item != NULL );
	
	if ( (index < vl->_position) && (index >= 0) )
		vl->_items[index] = (void *)item;
} /* }}} */

/* _vlist_index_of() - returns the position from the list of the given element {{{ */
_int32b _vlist_index_of( _vlist * vl, _vXhz_CONST void * item ) {
	_int32b result = -1, i;
	ASSERT( vl );
	
	for ( i = 0; i< vl->_position; i++ )
	{
		if ( vl->_items[i] == item )
		{ result = i; break; }
	}
	
	return result;
} /* }}} */

/* _vlist_insert() - inserts element into the list {{{ */
_int32b _vlist_insert( _vlist * vl, _vXhz_CONST void * item ) {
	ASSERT( vl );
	ASSERT( item != NULL );
	
	if ( vl->_position == (vl->_size - 1) )
	{
		void * _t =  malloc( sizeof( void * ) * (vl->_size + vl->_threshold) );      
		if ( _t == NULL )
			return out_of_memory;
		
		memcpy( _t, vl->_items, sizeof( void * ) * vl->_size );
		
		free( vl->_items ); 
		vl->_items = _t;
		
		memset( &(vl->_items[vl->_size]), 0, vl->_threshold );
		vl->_size += vl->_threshold;
	}
	
	vl->_items[vl->_position] = (void *)item;
	return vl->_position++;
} /* }}} */

/* _vlist_insert_at() {{{ */
void * _vlist_insert_at( _vlist * vl, _vXhz_CONST void * item, _int32b index ) {
	ASSERT( vl );
	ASSERT( item != NULL );
	
	/* TODO: IMPLEMENTATIO NEEDED */    
	return NULL;
} /* }}} */

/* _vlist_remove() {{{ */
_int32b _vlist_remove( _vlist * vl, _vXhz_CONST void * item ) {
	_int32b _i = -1;
	
	ASSERT( vl );
	ASSERT( item != NULL );
	
	if ( (_i = _vlist_index_of( vl, item )) >= 0 )
		_vlist_remove_at( vl, _i );
	
	return _i;
} /* }}} */

/* _vlist_remove_at() {{{ */
void * _vlist_remove_at( _vlist * vl, _int32b index ) {
	void * _vresult = NULL;
	
	ASSERT( vl );
	ASSERT( (index < vl->_position) && (index >= 0) );
	
	_vresult = _vlist_get_item_at( vl, index );
	
	if ( index == vl->_position - 1 )
		vl->_items[index] = NULL;
	else {
		memmove( &(vl->_items[index]), &(vl->_items[index + 1]), 
				sizeof( void * ) * (vl->_position - index - 1) );
	}
	
	vl->_position--;
	return _vresult;
} /* }}} */

/* _vlist_remove_all() {{{ */
void _vlist_remove_all( _vlist * vl ) {
	ASSERT( vl );
	
	memset( vl->_items, 0, sizeof( void * ) * vl->_position );
	vl->_position = 0;
} /* }}} */

/* _vlist_delete_all() {{{ */
void _vlist_delete_all( _vlist * vl, void (* clean_up)( void * ) ) {
	ASSERT( vl && clean_up );
	
	vl->_position = 0;
	/* TODO: IMPLEMENTATION NEEDED */
} /* }}} */

/* _vlist_swap() {{{ */
void _vlist_swap( _vlist * vl, _int32b i1, _int32b i2 ) {
	void * _t1, * _t2;
	
	ASSERT( vl );
	ASSERT( (i1 >= 0) && (i2 >= 0) && (i1 < vl->_position) && 
			(i2 < vl->_position) && (i1 != i2) );
	
	_t1 = _vlist_get_item_at( vl, i1 );
	_t2 = _vlist_get_item_at( vl, i2 );
	
	_vlist_set_item_at( vl, _t1, i2 );
	_vlist_set_item_at( vl, _t2, i1 );
} /* }}} */

/* _vlist_foreach() {{{ */
_status_t _vlist_foreach( _vlist * vl, _status_t (* test)( void *, void * ), void * args ) {
	_int32b  _idx;
	_status_t _return_code = OK;
	
	ASSERT( vl );
	ASSERT( test != NULL );
	
	_idx = 0;
	while ( _idx < vl->_position ) {
		_return_code = test( 
				_vlist_get_item_at( vl, _idx++ ), 
				args );
		
		if ( _return_code != OK )
			break;
	}
	
	return _return_code;
} /* }}} */



/* VIM Settings {{{
 * Local variables:
 * tab-width: 4
 * c-basic-offset: 4
 * soft-stop-width: 4
 * c indent on
 * End:
 * vim600: sw=4 ts=4 sts=4 cindent fdm=marker
 * vim<600: sw=4 ts=4
 * }}} */

