// $Id: main.h,v 1.25 2002/07/31 14:35:14 blues Exp $

/* Intro {{{
 * ---------------------------------------------------------------- 
 *  DirectConnect HUB for Linux
 * 
 *      #(@) Copyright (c) 2002, DConnect development team
 *      #(@) Homepage: http://www.dc.ds.pg.gda.pl/
 *
 *  This  program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License as
 *  published by the Free Software Foundation; either version 2 of
 *  the License.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY  or  FITNESS FOR A PARTICULAR PURPOSE. See the
 *  GNU General Public License for more details.
 *
 *  You  should  have  received a copy  of  the GNU General Public
 *  License  along  with  this  program;  if  not,  write  to  the
 *  Free  Software  Foundation,  Inc.,  675  Mass  Ave,  Cambridge,
 *  MA 02139, USA. 
 *
 * ---------------------------------------------------------------- 
 * }}} */

#if !defined( __vXhz__MAIN_H__INCLUDED__ )
#define __vXhz__MAIN_H__INCLUDED__ 

/* ChangeLog {{{
 * $Log: main.h,v $
 * Revision 1.25  2002/07/31 14:35:14  blues
 * - one TODO is out - it's possible now to turn off chats
 *
 * Revision 1.24  2002/07/30 16:15:10  blues
 * - all files have now vim helpers - in *.h files only Intro and ChangeLog
 *   are hidden. In *.c files all functions are hidden.
 * - added "/" at the end of address, fixed missing.
 *
 * Revision 1.23  2002/06/30 04:04:08  blues
 * - added MOTD utility
 *
 * Revision 1.22  2002/06/30 02:03:10  blues
 * - even more cleanups
 * - UID and GID are now config options
 * - status expanded
 *
 * Revision 1.21  2002/06/29 23:55:33  blues
 * - missing changes
 *
 * Revision 1.20  2002/06/29 23:50:59  blues
 * - massive attack:
 * - new logging scheme - used Log Writer Library
 * - autoconf stuff adapted partially
 * 
 * Revision 1.19  2002/06/24 20:28:49  blues
 * - killed buffer overflow
 * - BUF_SIZE grown to 4096
 * That should be 0.0.6 (as bugfix...)
 *
 * Revision 1.18  2002/06/06 11:03:46  blues
 * - add new line at the end of the file - causes warnings in RH
 *
 * Revision 1.17  2002/05/28 18:40:19  sebo
 * Simple config file parser and minimum share requirements.
 * Please view the changes and test it.
 *
 * Revision 1.16  2002/05/14 09:59:51  sebo
 * - empty users bugfix
 * - registration timeout introduced (default = 30sec.)
 * - a listen() backlog issue corrected
 *
 * Revision 1.15  2002/05/06 22:03:56  sebo
 * Ignoring too long lines instead of shutting down hub. Not tested 'cause I'm going to bed ;-)
 *
 * Revision 1.14  2002/05/06 18:56:38  blues
 * - Use project name as default name
 *
 * Revision 1.13  2002/05/06 18:54:35  blues
 * - use 1000 users by default
 *
 * Revision 1.12  2002/04/28 12:36:19  sebo
 * Daemon mode now works (I hope) and is the default. `-d' debug mode introduced.
 *
 * Revision 1.11  2002/04/27 18:21:10  ventox
 * Initial revision
 * 
 * }}} */

void adduser( int * sock, char * remote );

void deluser( int, char * );
void finish( int );

/* define default values
 * These are defined at compile time. */

/* Default configuration {{{ */

/* buffer size - shouldn't be touched	*/
#define BUF_SIZE 4096

/* daemon's listening port number	*/
#define DEFAULT_PORT 411

/* Run in deamon mode:
 * 1 = detach from tty, 
 * 0 = keep connected; 
 * can be overriden with -d
 */
#define DAEMON 1

/* max number of simultanously connected clients (for static buffers) */
#define MAXUSERS 1000

/* limit for number of users (dc protocol level); <=MAXUSERS (!!!) */
#define USERLIMIT 1000

/* name of our hub, obviously	*/
#define HUBNAME "DConnectDaemon"

/* set to 1, if hub should allow clients to connect even when they reply
 * with an invalid $Key string */
#define ALLOW_BROKEN_KEY 0

/* Allow chats or not - default yes */
#define ALLOW_CHAT 1

/* ping timeout duration in seconds; 0 = no timeout	*/
#define PING_TIMEOUT 0

/* registration time limit in seconds; 0 = no limit	*/
#define REGISTER_TIMEOUT 30

/* default path to config file	*/
#define CONFIG_FILE "dchub.conf"

/* required nick pattern	*/
#define NICK_PATTERN "*"

/* location of logfile	*/
#define LOG_DIR		"/var/log"
#define	DCD_LOG_FILE	"dcd.log"

/* default MOTD		*/
#define MOTD		"http://www.dc.ds.pg.gda.pl/"

/*  end of default values  }}} */

/* record with user's data */
struct userrec
{
 int sock;
 char state;
 char nick[20];
 char ip[16];
 char ver[16];
 long int ping;
 unsigned long long int share;
 char flag;
 char ctype[256];
 char desc[256];
 char email[256];
};

/* filled with config file data */
struct config
{
 char *file;
 char *log;
 char daemon;
 unsigned short int listen_port;
 char *hubname;
 char *motd;
 int userlimit;
 char allow_broken_key;
 char allow_chat;
 char ping_timeout;
 char register_timeout;
 char *nick_pattern;
 unsigned long long int minshare;
 int minslots;
 unsigned int uid_num;
 unsigned int gid_num;
};

#endif /* __vXhz__MAIN_H__INCLUDED__ */



/* VIM Settings {{{
 * Local variables:
 * tab-width: 4
 * c-basic-offset: 4
 * soft-stop-width: 4
 * c indent on
 * End:
 * vim600: sw=4 ts=4 sts=4 cindent fdm=marker
 * vim<600: sw=4 ts=4
 * }}} */

