static char rcs_id[] =
    "$Id: mutex.c,v 1.6 2002/07/30 16:15:10 blues Exp $";

/* Intro {{{
 * ---------------------------------------------------------------- 
 *  DConnect Daemon for Linux
 * 
 *      #(@) Copyright (c) 2002, DConnect development team
 *      #(@) Homepage: http://www.dc.ds.pg.gda.pl/
 *
 *  This  program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License as
 *  published by the Free Software Foundation; either version 2 of
 *  the License.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY  or  FITNESS FOR A PARTICULAR PURPOSE. See the
 *  GNU General Public License for more details.
 *
 *  You  should  have  received a copy  of  the GNU General Public
 *  License  along  with  this  program;  if  not,  write  to  the
 *  Free  Software  Foundation,  Inc.,  675  Mass  Ave,  Cambridge,
 *  MA 02139, USA. 
 *
 * ---------------------------------------------------------------- 
 * }}} */

// include for PCH file
#include "pch.h"

// include local header files
#include "mutex.h"

#if defined( _DEBUG )
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif /* _DEBUG */

/* helper macros  {{{ */

#define _MUTEX_LOCK( _m ) \
  { \
    ASSERT( (_vmutex_t)(_m) != NULL ); \
  \
    pthread_mutex_lock( \
      &((_vmutex_t)(_m))->_handle ); \
  } \

#define _MUTEX_UNLOCK( _m ) \
  { \
    ASSERT( (_vmutex_t)(_m) != NULL ); \
  \
    pthread_mutex_unlock( \
      &((_vmutex_t)(_m))->_handle ); \
  } \
/* }}} */

/* functions implementation */

/* _vmutex_init() - function creates a named or unnamed mutex object {{{ */
_status_t _vmutex_init( _vmutex_t * _mutex, BOOL owned ) { 
	_status_t result = 0;
	_vmutex_t mutex;
	
	ASSERT( _mutex );
	
	mutex = (_vmutex_t)
		calloc( 1, sizeof( struct _tagMUTEX ) );
	
	if ( mutex == NULL ) {
		result = out_of_memory;
	}
	else {
		pthread_mutexattr_t _attr;
		
		mutex->_owner  = (pthread_t)NULL;      
		mutex->_busy   = 0;
		
		pthread_cond_init( &mutex->_cond, NULL );
		pthread_mutexattr_init( &_attr );    
		
		if ( !pthread_mutex_init( &mutex->_handle, &_attr ) ) {
			if ( owned )
				_MUTEX_LOCK( mutex );
		}
		
		pthread_mutexattr_destroy( &_attr );
	}
	
	if ( result == OK )
		(*_mutex) = mutex;
	
	return result;
} // }}}

/* _vmutex_destroy() - function destroys the specyfied mutex object {{{ */
_status_t _vmutex_destroy( _vmutex_t * _mutex ) {
	_status_t result = OK;
	ASSERT( _mutex );
	
	if ( (*_mutex) == NULL ) {
		result = invalid_argument;
	}
	else {
		pthread_cond_destroy( &(*_mutex)->_cond  );
		pthread_mutex_destroy( &(*_mutex)->_handle );            
		
		free( *_mutex ); *_mutex = NULL;
	}
	
	return result;
} // }}}

/* _vmutex_acquire() - function obtains access to the owner of the specified mutex object {{{ */
_status_t _vmutex_acquire( _vmutex_t * _mutex, _int32b timeout ) {
	_MUTEX_LOCK( *_mutex );
	
	if ( (*_mutex)->_busy != 0 ) {
		if ( (*_mutex)->_owner != pthread_self() ) {
			do {
				if ( timeout == WAIT_FOREVER )
					pthread_cond_wait( &(*_mutex)->_cond, &(*_mutex)->_handle );
				else {
					/* not yet implemented */
				}
			} while ( (*_mutex)->_busy != 0 );
		}
	}
	(*_mutex)->_owner = pthread_self();
	(*_mutex)->_busy++;
	
	_MUTEX_UNLOCK( *_mutex );
	return OK;
} // }}}

/* _vmutex_release() - function releases ownership of the specyfied  mutex object {{{ */
_status_t _vmutex_release( _vmutex_t * _mutex ) {
	int _t;
	
	_MUTEX_LOCK( *_mutex );
	_t = --(*_mutex)->_busy;
	_MUTEX_UNLOCK( *_mutex );
	
	if ( _t == 0 )
		pthread_cond_signal( &(*_mutex)->_cond );
	
	return OK;
} // }}}


/* VIM Settings {{{
 * Local variables:
 * tab-width: 4
 * c-basic-offset: 4
 * soft-stop-width: 4
 * c indent on
 * End:
 * vim600: sw=4 ts=4 sts=4 cindent fdm=marker
 * vim<600: sw=4 ts=4
 * }}} */

