static char rcs_id[] =
	"$Id: net.c,v 1.17 2002/07/30 16:15:10 blues Exp $";

/* Intro {{{
 * ---------------------------------------------------------------- 
 *  DirectConnect HUB for Linux
 * 
 *      #(@) Copyright (c) 2002, DConnect development team
 *      #(@) Homepage: http://www.dc.ds.pg.gda.pl/
 *
 *  This  program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License as
 *  published by the Free Software Foundation; either version 2 of
 *  the License.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY  or  FITNESS FOR A PARTICULAR PURPOSE. See the
 *  GNU General Public License for more details.
 *
 *  You  should  have  received a copy  of  the GNU General Public
 *  License  along  with  this  program;  if  not,  write  to  the
 *  Free  Software  Foundation,  Inc.,  675  Mass  Ave,  Cambridge,
 *  MA 02139, USA. 
 *
 * error.c: interface for handling errors from library functions
 * ---------------------------------------------------------------- 
 * }}} */

/* include for PCH file */
#include "pch.h"

/* ChangeLog {{{
 * $Log: net.c,v $
 * Revision 1.17  2002/07/30 16:15:10  blues
 * - all files have now vim helpers - in *.h files only Intro and ChangeLog
 *   are hidden. In *.c files all functions are hidden.
 * - added "/" at the end of address, fixed missing.
 *
 * Revision 1.16  2002/07/20 20:01:13  blues
 * - some helper functions for vim
 *
 * Revision 1.15  2002/06/30 06:44:10  blues
 * - more logging
 *
 * Revision 1.14  2002/06/30 03:35:28  blues
 * - bugfix
 *
 * Revision 1.13  2002/06/30 03:33:11  blues
 * - more logging
 *
 * Revision 1.12  2002/06/30 03:20:13  blues
 * - cleanups
 *
 * Revision 1.11  2002/06/30 02:03:10  blues
 * - even more cleanups
 * - UID and GID are now config options
 * - status expanded
 *
 * Revision 1.10  2002/06/29 23:50:59  blues
 * - massive attack:
 * - new logging scheme - used Log Writer Library
 * - autoconf stuff adapted partially
 *
 * Revision 1.9  2002/06/24 20:28:49  blues
 * - killed buffer overflow
 * - BUF_SIZE grown to 4096
 * That should be 0.0.6 (as bugfix...)
 *
 * Revision 1.8  2002/06/24 19:50:09  blues
 * - few cosmetic fixes, nothing to care about.
 *
 * Revision 1.7  2002/05/28 18:40:19  sebo
 * Simple config file parser and minimum share requirements.
 * Please view the changes and test it.
 *
 * Revision 1.6  2002/04/27 18:21:10  ventox
 * Initial revision
 *
 * }}} */

/* include local header files		*/
#include "main.h"

extern struct userrec *user[];
extern int NUSERS;
extern lwlh_t h;

/* send_to() - send to single user {{{ */
int send_to( struct userrec * usr, char * str ) {
	printf( "->[%s] %s\n", *(usr->nick) ? usr->nick : "Unknown", str ); 
	lwl_write_log (h, LWL_PRI_INFO, "->[%s] %s", *(usr->nick)?usr->nick:"Unknown", str );
	fflush( stdout );
	
	strcat( str, "|" );
	lwl_write_log (h, LWL_PRI_DEBUG, "->[%s] %s", *(usr->nick)?usr->nick:"Unknown", str );
	return send( usr->sock, str,strlen( str ), 0 );
} // }}}

/* send_mcast() - send multicast {{{ */
int send_mcast( char *str ) {
	int n;
	
	printf( "->[*] %s\n", str ); 
	lwl_write_log (h, LWL_PRI_INFO, "->[*] %s", str );
	fflush( stdout );
	
	strcat( str, "|" );
	lwl_write_log (h, LWL_PRI_DEBUG, "->[*] %s", str );
	for ( n = 0; n < NUSERS; n++ ) 
		send( user[n]->sock, str, strlen( str ), 0 );
	
	return 0;
} // }}}

/* sendf_to() - something like printf(), but to a socket - one destination {{{ */
int sendf_to( struct userrec * usr, char * format, ... ) {
	char str[4096];
	va_list args;
	
	va_start( args, format );
	vsnprintf( str, 4095, format, args );
	va_end( args );
	
	printf( "->[%s] %s\n", *(usr->nick) ? usr->nick : "Unknown", str ); 
	lwl_write_log (h, LWL_PRI_INFO, "->[%s] %s", *(usr->nick)?usr->nick:"Unknown", str );
	fflush(stdout);
	
	strcat(str,"|");
	lwl_write_log (h, LWL_PRI_DEBUG, "->[%s] %s", *(usr->nick)?usr->nick:"Unknown", str );
	return send( usr->sock, str, strlen( str ), 0 );
} // }}}

/* sendf_mcast() - formated send, multicast {{{ */
int sendf_mcast( char * format, ...) {
	int n;
	char str[4096];
	va_list args;
	
	va_start( args, format );
	
	vsnprintf( str, 4095, format, args );
	va_end( args );
	
	printf( "->[*] %s\n", str ); 
	lwl_write_log (h, LWL_PRI_INFO, "->[*] %s", str );
	fflush( stdout );
	
	strcat( str, "|" );
	lwl_write_log (h, LWL_PRI_DEBUG, "->[*] %s", str );
	for ( n = 0; n < NUSERS; n++ ) 
		send( user[n]->sock, str, strlen( str ), 0 );
	
	return 0;
} // }}}

/* this is required, since after disconnect a socket is switched to a TIME_WAIT
 * state and the listening port remains occupied for several minutes */
/* setsockopts() {{{ */
int setsockopts(int *sock) {
	int flag=1,ret=1;
	struct linger linger;
	
	linger.l_onoff=0; linger.l_linger=0;
	if (setsockopt(*sock,SOL_SOCKET,SO_REUSEADDR,&flag,sizeof(flag))<0) ret=0;
	if (setsockopt(*sock,SOL_SOCKET,SO_LINGER,&linger,sizeof(linger))<0) ret=0;
	if (setsockopt(*sock,SOL_SOCKET,SO_KEEPALIVE,&flag,sizeof(flag))<0) ret=0;
	return ret;
} // }}}

/* publicmsg () - send message to public chat window {{{ */
void publicmsg(char *from,struct userrec *to,char *msg) {
	char who[20];
	if (from) strcpy(who,from); else strcpy(who,"Hub-Security");
	if (to) sendf_to(to,"<%s> %s",who,msg);
	else sendf_mcast("<%s> %s",who,msg);
} // }}}

/* privmsg() - send message to private chat {{{ */
void privmsg(char *from,struct userrec *to,char *msg) {
	if (!to) return;
	if (from) sendf_to(to,"$To: %s From: %s $<%s> %s",to->nick,from,from,msg);
	else sendf_to(to,"$To: %s From: Hub $<Hub> %s",to->nick,msg);
} // }}}

/* disconnect() - disconnect a user with optional reason message {{{ */
void disconnect(struct userrec *usr,char *reason) {
	if (reason) publicmsg(NULL,usr,reason);
	close(usr->sock);
	printf("Disconnected %s",*(usr->nick)?usr->nick:"Unknown");
	if (reason) printf(": %s\n",reason); else printf(".\n");
	lwl_write_log (h, LWL_PRI_INFO, "Disconnected %s (reason: %s)", *(usr->nick)?usr->nick:"Unknown", reason);
	free(usr);
} // }}}


/* VIM Settings {{{
 * Local variables:
 * tab-width: 4
 * c-basic-offset: 4
 * soft-stop-width: 4
 * c indent on
 * End:
 * vim600: sw=4 ts=4 sts=4 cindent fdm=marker
 * vim<600: sw=4 ts=4
 * }}} */

