static char rcs_id[] =
    "$Id: semaphore.c,v 1.4 2002/07/30 16:15:10 blues Exp $";

/* Intro {{{
 * ---------------------------------------------------------------- 
 *  DConnect Daemon for Linux
 * 
 *      #(@) Copyright (c) 2002, DConnect development team
 *      #(@) Homepage: http://www.dc.ds.pg.gda.pl/
 *
 *  This  program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License as
 *  published by the Free Software Foundation; either version 2 of
 *  the License.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY  or  FITNESS FOR A PARTICULAR PURPOSE. See the
 *  GNU General Public License for more details.
 *
 *  You  should  have  received a copy  of  the GNU General Public
 *  License  along  with  this  program;  if  not,  write  to  the
 *  Free  Software  Foundation,  Inc.,  675  Mass  Ave,  Cambridge,
 *  MA 02139, USA. 
 *
 * ---------------------------------------------------------------- 
 * }}} */

/* ChangeLog {{{
 * $Log: semaphore.c,v $
 * Revision 1.4  2002/07/30 16:15:10  blues
 * - all files have now vim helpers - in *.h files only Intro and ChangeLog
 *   are hidden. In *.c files all functions are hidden.
 * - added "/" at the end of address, fixed missing.
 *
 * Revision 1.3  2002/06/30 07:45:32  blues
 * - finish of the cosmetics
 *
 * Revision 1.2  2002/05/28 18:40:19  sebo
 * Simple config file parser and minimum share requirements.
 * Please view the changes and test it.
 *
 * Revision 1.1  2002/05/18 20:08:22  ventox
 * - initial revision
 * }}} */

// include for PCH file
#include "pch.h"

// include system depended header files
#if defined( WIN32 )
#include <sys/timeb.h>
#endif /* WIN32 */

// include local header files
#include "mutex.h"
#include "semaphore.h"

#if defined( _DEBUG )
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif /* _DEBIG */

// function initializes an unnamed semaphore object
_status_t _vsmphr_init( _vsmphr_t * _smphr, uint32b value ) {
	_status_t result = OK;
	_vsmphr_t smphr;
	
	ASSERT( _smphr );
	
	smphr = (_vsmphr_t)
		calloc( 1, sizeof( struct _tagSMPHR ) );
	
	if ( smphr == NULL ) {
		result = out_of_memory;
	}
	else {
#if defined( WIN32 )
		
		smphr->handle = 
			CreateSemaphore( NULL, value, INT_MAX, NULL );
		
		if ( 0 == smphr->handle )
			result = resource_exhausted;
		
#else /* WIN32 */
		
		pthread_mutex_init( 
				&smphr->_handle, NULL );
		
		pthread_cond_init ( 
				&smphr->_cond  , NULL );
		
		smphr->_value = value;
		
#endif /* !WIN32 */
	}
	
	if ( result == OK )
		(*_smphr) = smphr;
	
	return result;
}

// functions destroys the specyfied semaphore object
_status_t _vsmphr_destroy( _vsmphr_t * _smphr ) {
	_status_t result = OK;
	ASSERT( _smphr );
	
	if ( (*_smphr) == NULL ) {
		result = invalid_argument;
	}
	else {
#if defined( WIN32 )
		
		if ( !CloseHandle( (*_smphr)->handle ) ) {
			result = invalid_argument;
		}
		
#else /* WIN32 */
		
		pthread_mutex_destroy( &(*_smphr)->_handle );
		pthread_cond_destroy ( &(*_smphr)->_cond  );
		
#endif /* !WIN32 */
	}
	
	if ( result == OK ) {
		free( (void *)(*_smphr) );
		(*_smphr) = NULL;
	}
	
	return result;
}

#define _COND_WAIT( _smphr ) \
  { \
    pthread_cond_wait( \
      &(*(_vsmphr_t *)(_smphr))->_cond, \
      &(*(_vsmphr_t *)(_smphr))->_handle ); \
  } \
    
/* functions returns when the specyfied object signaled state
 * or timeout interval elapses */
_status_t _vsmphr_wait( _vsmphr_t * _smphr, _int32b timeout ) {
	_status_t result = OK;
	ASSERT( _smphr );
	
	if ( (*_smphr) == NULL ) {
		return invalid_argument;
	}
	
	if ( timeout > 0 ) {
		/* result = _vsmphr_timed_wait( _smphr, timeout );  */
	}
	else {
#if defined( WIN32 )
		
		result = pthreadCancelableWait( (*_smphr)->handle );
		
#else /* WIN32 */
		
		pthread_mutex_lock( &(*_smphr)->_handle );
		
		(*_smphr)->_value--;
		while ( (*_smphr)->_value <= 0 )
			_COND_WAIT( _smphr );
		
		pthread_mutex_unlock( &(*_smphr)->_handle );
		
#endif /* !WIN32 */
	}
	
	return result;
}

#define NANOSEC_PER_MILLISEC   ((DWORD)1000000)
#define MILLISEC_PER_SEC       ((DWORD)1000)

// function waits until the specyfied period of the time elapses
_status_t _vsmphr_timed_wait( _vsmphr_t * _smphr, _int32b thetime ) {
	_status_t result = OK;
	ASSERT( _smphr );
	
	if ( (*_smphr) == NULL ) {
		result = invalid_argument;
	}
	else {
#if defined( WIN32 )
		
		/* need to be fixed */
		if ( WaitForSingleObject( 
					(*_smphr)->handle, 
					thetime ) == WAIT_TIMEOUT ) {
			result = time_out;
		}

#else /* WIN32 */
		
		// not yet implemented
		
#endif /* !WIN32 */
	}
	
	return result;
}

/* function changes the state of the specyfied semaphore to
 * its signaled state  */
_status_t _vsmphr_signal( _vsmphr_t * _smphr ) {
	_status_t result = OK;
	ASSERT( _smphr );
	
	if ( (*_smphr) == NULL ) {
		result = invalid_argument;
	}
	else {
#if defined( WIN32 )
		
		long _dummy;
		ReleaseSemaphore( (*_smphr)->handle, 1, &_dummy );        
		
#else /* WIN32 */
		
		// not yet implemented
		
#endif /* !WIN32 */
	}
	
	return result;
}


/* VIM Settings {{{
 * Local variables:
 * tab-width: 4
 * c-basic-offset: 4
 * soft-stop-width: 4
 * c indent on
 * End:
 * vim600: sw=4 ts=4 sts=4 cindent fdm=marker
 * vim<600: sw=4 ts=4
 * }}} */

