static char rcs_id[] =
    "$Id: thread.c,v 1.4 2002/07/30 16:15:10 blues Exp $";

/* Intro {{{
 * ---------------------------------------------------------------- 
 *  DConnect Daemon for Linux
 * 
 *      #(@) Copyright (c) 2002, DConnect development team
 *      #(@) Homepage: http://www.dc.ds.pg.gda.pl/
 *
 *  This  program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License as
 *  published by the Free Software Foundation; either version 2 of
 *  the License.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY  or  FITNESS FOR A PARTICULAR PURPOSE. See the
 *  GNU General Public License for more details.
 *
 *  You  should  have  received a copy  of  the GNU General Public
 *  License  along  with  this  program;  if  not,  write  to  the
 *  Free  Software  Foundation,  Inc.,  675  Mass  Ave,  Cambridge,
 *  MA 02139, USA. 
 *
 * ---------------------------------------------------------------- 
 * }}} */

/* ChangeLog {{{
 * $Log: thread.c,v $
 * Revision 1.4  2002/07/30 16:15:10  blues
 * - all files have now vim helpers - in *.h files only Intro and ChangeLog
 *   are hidden. In *.c files all functions are hidden.
 * - added "/" at the end of address, fixed missing.
 *
 * Revision 1.3  2002/06/30 07:51:00  blues
 * - switched to our main web page
 *
 * Revision 1.2  2002/05/28 18:40:19  sebo
 * Simple config file parser and minimum share requirements.
 * Please view the changes and test it.
 *
 * Revision 1.1  2002/05/18 20:08:22  ventox
 * - initial revision
 * }}} */

// include for PCH file
#include "pch.h"

// include local header files
#include "mutex.h"
#include "semaphore.h"

#include "thread.h"

#if !defined( _DEBUG )
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif /* _DEBUG */

static void * _internal_thread_proc( void * thread )
  {
    _vthread_t _thrd = 
      (_vthread_t)thread;

    ASSERT( thread );

    _vthread_WAIT( _thrd );
    
    if ( _thrd->_t_proc )
      _thrd->_t_proc( _thrd );

    return NULL;
  }

// function creates a thread
_status_t _vthread_init( _vthread_t * _thrd, 
  _vthread_proc t_proc, void * args, BOOL detach )
  {
    _status_t result = OK;
    ASSERT( _thrd );

    if ( t_proc == NULL )
      {
        result = invalid_thread_proc;
      }
    else
      {
        _vthread_t thread;        
        
        thread = (_vthread_t)
          calloc( 1, sizeof( _vthread ) );

        if ( thread == NULL )
          {
            result = out_of_memory;            
          }
        else
          {
            _vmutex_init( &thread->_mutex, FALSE );
            _vsmphr_init( &thread->_semaphore, _SMPHR_LOCKED );

            thread->_handle    = (pthread_t)NULL;
            thread->_id        = 0;
            thread->_suspended = FALSE;       
            thread->_args      = args;
            thread->_t_proc    = t_proc;

            if ( pthread_create( &thread->_handle, 
              NULL, _internal_thread_proc, thread ) )
              {
//                _vthread_destroy( &thread );
                result = init_failed;
              }
            else
              {
                if ( detach )
                  pthread_detach( thread->_handle );

                _vthread_SIGNAL( thread );
              }
          }

        if ( result == OK )
          (*_thrd) = thread;
      }

    return result;
  }

// function destroys thread and cleans up memory
_status_t _vthread_destroy( _vthread_t * _thrd )
  {
    _status_t result = OK;
    ASSERT( _thrd );

    if ( (*_thrd) == NULL )
      {
        result = invalid_argument;
      }
    else
      {
        pthread_cancel (  (*_thrd)->_handle );        
    
        _vsmphr_destroy( &(*_thrd)->_semaphore );    
        _vmutex_destroy( &(*_thrd)->_mutex );    
    
        free( (void *)(*_thrd) );
        (*_thrd) = NULL;
      }

    return result;
  }

_status_t _vthread_join( _vthread_t * _thrd )
  {
    _status_t result = OK;
    ASSERT( _thrd );

    if ( (*_thrd) == NULL )
      {
        result = invalid_argument;
      }
    else
      {
        pthread_join( (*_thrd)->_handle, NULL );
      }

    return result;
  }

// function suspends the specyfied thread
_status_t _vthread_suspend( _vthread_t * _thrd )
  {
    _status_t result = OK;
    ASSERT( _thrd );

    if ( (*_thrd) == NULL )
      {
        result = invalid_argument;
      }
    else
      {
#if defined( WIN32 )

        if ( !(*_thrd)->_suspended )
          {
            (*_thrd)->_suspended = TRUE;
            _vthread_WAIT( *_thrd );
          }

#else /* WIN32 */

        _vthread_ACQUIRE( *_thrd );
        if ( !(*_thrd)->_suspended )
        {
          pthread_kill( (*_thrd)->_handle, SIGSTOP );
          (*_thrd)->_suspended = TRUE;
        }
        _vthread_RELEASE( *_thrd );

#endif /* WIN32 */
      }

    return result;
  }

// function decrements a thread's suspend count
_status_t _vthread_resume( _vthread_t * _thrd )
  {
    _status_t result = OK;
    ASSERT( _thrd );

    if ( (*_thrd) == NULL )
      {
        result = invalid_argument;
      }
    else
      {
#if defined( WIN32 )
        
        if ( (*_thrd)->_suspended )
          {
            (*_thrd)->_suspended = FALSE;
            _vthread_SIGNAL( *_thrd );
          }

#else /* WIN32 */

        _vthread_ACQUIRE( *_thrd );
        if ( (*_thrd)->_suspended )
        {
          pthread_kill( (*_thrd)->_handle, SIGCONT );
          (*_thrd)->_semaphore = FALSE;
        }
        _vthread_RELEASE( *_thrd );

#endif /* !WIN32 */
      }
   
    return result;
  }

// function returns the priority of the specyfied thread
_int16b _vthread_get_priority( _vthread_t * _thrd )
  {
    _status_t result = OK;
    ASSERT( _thrd );

    // not yet implemented
    return (-1);
  }

// function sets the priority value for the specyfied thread
_status_t _vthread_set_priority( _vthread_t * _thrd, _int16b value )
  {
    _status_t result = OK;
    ASSERT( _thrd );

    if ( (*_thrd) == NULL )
      {
        result = invalid_argument;
      }
    else
      {
        // not yet implemented
      }

    return OK;
  }



/* VIM Settings {{{
 * Local variables:
 * tab-width: 4
 * c-basic-offset: 4
 * soft-stop-width: 4
 * c indent on
 * End:
 * vim600: sw=4 ts=4 sts=4 cindent fdm=marker
 * vim<600: sw=4 ts=4
 * }}} */

