// $Id: thread.h,v 1.5 2002/07/30 16:15:10 blues Exp $

/* Intro {{{
 * ---------------------------------------------------------------- 
 *  DConnect Daemon for Linux
 * 
 *      #(@) Copyright (c) 2002, DConnect development team
 *      #(@) Homepage: http://www.dc.ds.pg.gda.pl/
 *
 *  This  program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License as
 *  published by the Free Software Foundation; either version 2 of
 *  the License.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY  or  FITNESS FOR A PARTICULAR PURPOSE. See the
 *  GNU General Public License for more details.
 *
 *  You  should  have  received a copy  of  the GNU General Public
 *  License  along  with  this  program;  if  not,  write  to  the
 *  Free  Software  Foundation,  Inc.,  675  Mass  Ave,  Cambridge,
 *  MA 02139, USA. 
 *
 * ---------------------------------------------------------------- 
 * }}} */

/* ChangeLog {{{
 * $Log: thread.h,v $
 * Revision 1.5  2002/07/30 16:15:10  blues
 * - all files have now vim helpers - in *.h files only Intro and ChangeLog
 *   are hidden. In *.c files all functions are hidden.
 * - added "/" at the end of address, fixed missing.
 *
 * Revision 1.4  2002/06/30 07:51:00  blues
 * - switched to our main web page
 *
 * Revision 1.3  2002/06/06 11:03:46  blues
 * - add new line at the end of the file - causes warnings in RH
 *
 * Revision 1.2  2002/05/28 18:40:19  sebo
 * Simple config file parser and minimum share requirements.
 * Please view the changes and test it.
 *
 * Revision 1.1  2002/05/18 20:08:22  ventox
 * - initial revision
 * }}} */

#if !defined( __vXhz__THREAD_H__INCLUDED__ )
#define __vXhz__THREAD_H__INCLUDED__ 

// thread handle proc
typedef void (_vXhz_STDCALL * _vthread_proc)( void * );

// thread 
typedef struct _tagTHREAD 
  {
    pthread_t     _handle;  
    _vthread_proc _t_proc; 
    BOOL          _suspended;
    uint32b       _id;
    void *        _args;
    _vsmphr_t     _semaphore;
    _vmutex_t     _mutex;
  } 
  _vthread, * _vthread_t;

#if defined( __cplusplus )
extern "C" {
#endif /* __cplusplus */

// function creates a thread
//_vXhz_LIB_ENTRY _status_t _vthread_init( _vthread_t * _thrd, _vthread_proc _thrd_proc, void * args, BOOL _detach );

#define _vthread_INIT( _thrd, _proc, _args ) \
  _vthread_init( (_vthread_t *)&(_thrd), \
    (_vthread_proc)_proc, (void *)_args, FALSE )

#define _vthread_INIT_AND_DETACH( _p, _a ) \
  _vthread_init( (_vthread_t *)&(_thrd), \
    (_vthread_proc)_proc, (void *)_args, TRUE )

// function cleans up memory allocated for a thread's data struct
//_vXhz_LIB_ENTRY _status_t _vthread_destroy( _vthread_t * _thrd );

#define _vthread_DESTROY( _thrd ) \
  _vthread_destroy( (_vthread_t *)&(_thrd) )

//_vXhz_LIB_ENTRY _status_t _vthread_join( _vthread_t * _thrd );

#define _vthread_JOIN( _thrd ) \
  _vthread_join( (_vthread_t *)&(_thrd) );

// function suspends the specyfied thread
//_vXhz_LIB_ENTRY _status_t _vthread_suspend( _vthread_t * _thrd );

#define _vthread_SUSPEND( _thrd ) \
  _vthread_suspend( (_vthread_t *)&(_thrd) )

// function decrements a thread's suspend count 
// (if it reaches '0' the execution of the thread is resumed)
//_vXhz_LIB_ENTRY _status_t _vthread_resume( _vthread_t * _thrd );

#define _vthread_RESUME( _thrd ) \
  _vthread_resume( (_vthread_t *)&(_thrd) )

// function sets the priority value for the specyfied thread
//_vXhz_LIB_ENTRY _status_t _vthread_set_priority( _vthread_t * _thrd, _int16b value );

// function returns the priority of the specyfied thread
//_vXhz_LIB_ENTRY _int16b _vthread_get_priority( _vthread_t * _thrd );

/* helper macros */

#define _vthread_WAIT( _thrd ) \
  { \
    ASSERT( (_vthread_t)(_thrd) != NULL ); \
    _vsmphr_WAIT( \
      ((_vthread_t)(_thrd))->_semaphore ); \
  } \

#define _vthread_SIGNAL( _thrd ) \
  { \
    ASSERT( (_vthread_t)(_thrd) != NULL ); \
    _vsmphr_SIGNAL( \
      ((_vthread_t)(_thrd))->_semaphore ); \
  } \

#define _vthread_ACQUIRE( _thrd ) \
  { \
    ASSERT( (_vthread_t)(_thrd) != NULL ); \
    _vmutex_ACQUIRE( \
      ((_vthread_t)(_thrd))->_mutex ); \
  } \

#define _vthread_RELEASE( _thrd ) \
  { \
    ASSERT( (_vthread_t)(_thrd) != NULL ); \
    _vmutex_RELEASE( \
      ((_vthread_t)(_thrd))->_mutex ); \
  } \

#define _vthread_SLEEP( _thrd, _time ) \
  { \
    ASSERT( (_vthread_t)(_thrd) != NULL ); \
    _vsmphr_TIMED_WAIT( \
      ((_vthread_t)(_thrd))->_semaphore, (_int32b)(_time) ); \
  } \

#if defined( __cplusplus )
}
#endif /* __cplusplus */

#endif /* __vXhz__MT_H__INCLUDED__ */



/* VIM Settings {{{
 * Local variables:
 * tab-width: 4
 * c-basic-offset: 4
 * soft-stop-width: 4
 * c indent on
 * End:
 * vim600: sw=4 ts=4 sts=4 cindent fdm=marker
 * vim<600: sw=4 ts=4
 * }}} */

